@@ -300,7 +300,7 @@ def _serialize_date(dt):
         dt = dt.timetuple()
     if isinstance(dt, (tuple, time.struct_time)):
         dt = calendar.timegm(dt)
-    if not isinstance(dt, (float, int)):
+    if not isinstance(dt, (float, int, long)):
         raise ValueError(
             "You must pass in a datetime, date, time tuple, or integer object, not %r" % dt)
     return formatdate(dt)
@@ -857,14 +857,16 @@ class Request(object):
 
     def str_POST(self):
         """
-        Return a MultiDict containing all the variables from a POST
-        form request.  Does *not* return anything for non-POST
-        requests or for non-form requests (returns empty dict-like
-        object in that case).
+        Return a MultiDict containing all the variables from a form
+        request. Returns an empty dict-like object for non-form
+        requests.
+
+        Form requests are typically POST requests, however PUT requests
+        with an appropriate Content-Type are also supported.
         """
         env = self.environ
-        if self.method != 'POST':
-            return NoVars('Not a POST request')
+        if self.method not in ('POST', 'PUT'):
+            return NoVars('Not a form request')
         if 'webob._parsed_post_vars' in env:
             vars, body_file = env['webob._parsed_post_vars']
             if body_file is self.body_file:
@@ -879,8 +881,9 @@ class Request(object):
         content_type = self.content_type
         if ';' in content_type:
             content_type = content_type.split(';', 1)[0]
-        if content_type not in ('', 'application/x-www-form-urlencoded',
-                                'multipart/form-data'):
+        if (self.method == 'PUT' and not content_type) or \
+                content_type not in ('', 'application/x-www-form-urlencoded',
+                                     'multipart/form-data'):
             # Not an HTML form submission
             return NoVars('Not an HTML form submission (Content-Type: %s)'
                           % content_type)
@@ -1119,7 +1122,7 @@ class Request(object):
         if remove_range:
             check_keys += ['HTTP_IF_RANGE', 'HTTP_RANGE']
         if remove_match:
-            check_keys.append('HTTP_IF_MATCH')
+            check_keys.append('HTTP_IF_NONE_MATCH')
         if remove_modified:
             check_keys.append('HTTP_IF_MODIFIED_SINCE')
         if remove_encoding:
@@ -1673,6 +1676,12 @@ class Response(object):
             raise TypeError(
                 "You can only set the body to a str (not %s)"
                 % type(value))
+        try:
+            if self._body or self._app_iter:
+                self.content_md5 = None
+        except AttributeError:
+            # if setting body early in initialization _body and _app_iter don't exist yet
+            pass
         self._body = value
         self.content_length = len(value)
         self._app_iter = None
@@ -1766,7 +1775,7 @@ class Response(object):
         cookies = BaseCookie()
         cookies[key] = value
         if isinstance(max_age, timedelta):
-            max_age = timedelta.seconds + timedelta.days*24*60*60
+            max_age = max_age.seconds + max_age.days*24*60*60
         if max_age is not None and expires is None:
             expires = datetime.utcnow() + timedelta(seconds=max_age)
         if isinstance(expires, timedelta):
@@ -1786,6 +1795,10 @@ class Response(object):
             if var_value is not None and var_value is not False:
                 cookies[key][var_name.replace('_', '-')] = str(var_value)
         header_value = cookies[key].output(header='').lstrip()
+        if header_value.endswith(';'):
+            # Python 2.4 adds a trailing ; to the end, strip it to be
+            # consistent with 2.5
+            header_value = header_value[:-1]
         self.headerlist.append(('Set-Cookie', header_value))
 
     def delete_cookie(self, key, path='/', domain=None):
@@ -1820,6 +1833,10 @@ class Response(object):
                 del cookies[key]
                 header = cookies.output(header='').lstrip()
             if header:
+                if header.endswith(';'):
+                    # Python 2.4 adds a trailing ; to the end, strip it
+                    # to be consistent with 2.5
+                    header = header[:-1]
                 self.headers.add('Set-Cookie', header)
         if not found:
             raise KeyError(
@@ -1954,6 +1971,7 @@ class Response(object):
         identity are supported).
         """
         if encoding == 'identity':
+            self.decode_content()
             return
         if encoding != 'gzip':
             raise ValueError(
@@ -2012,13 +2030,14 @@ class Response(object):
 
     etag = header_getter('ETag', rfc_section='14.19')
 
-    def md5_etag(self, body=None, set_content_md5=False):
+    def md5_etag(self, body=None, set_content_md5=False, set_conditional_response=False):
         """
         Generate an etag for the response object using an MD5 hash of
         the body (the body parameter, or ``self.body`` if not given)
 
         Sets ``self.etag``
-        If set_content_md5 is True sets self.content_md5 as well
+        If ``set_content_md5`` is True sets ``self.content_md5`` as well
+        If ``set_conditional_response`` is True sets ``self.conditional_response`` to True
         """
         if body is None:
             body = self.body
@@ -2028,6 +2047,8 @@ class Response(object):
         self.etag = md5_digest
         if set_content_md5:
             self.content_md5 = md5_digest
+        if set_conditional_response:
+            self.conditional_response = True
 
     expires = converter(
         header_getter('Expires', rfc_section='14.21'),
